@@ -79,7 +79,7 @@ camera_get_raw(pgCameraObject *self, PyObject *args);
 PyObject *
 surf_colorspace(PyObject *self, PyObject *arg)
 {
-    PyObject *surfobj, *surfobj2;
+    pgSurfaceObject *surfobj, *surfobj2;
     SDL_Surface *surf, *newsurf;
     char *color;
     int cspace;
@@ -136,10 +136,10 @@ surf_colorspace(PyObject *self, PyObject *arg)
 
     if (surfobj2) {
         Py_INCREF(surfobj2);
-        return surfobj2;
+        return (PyObject *)surfobj2;
     }
     else
-        return pgSurface_New(newsurf);
+        return (PyObject *)pgSurface_New(newsurf);
 }
 
 /* list_cameras() - lists cameras available on the computer */
@@ -166,7 +166,16 @@ list_cameras(PyObject *self, PyObject *arg)
 
     for (i = 0; i < num_devices; i++) {
         string = Text_FromUTF8(devices[i]);
-        PyList_Append(ret_list, string);
+        if (0 != PyList_Append(ret_list, string)) {
+            /* Append failed; clean up and return */
+            Py_DECREF(ret_list);
+            Py_DECREF(string);
+            for (; i < num_devices ; i++) {
+                free(devices[i]);
+            }
+            free(devices);
+            return NULL; /* Exception already set. */
+        }
         Py_DECREF(string);
         free(devices[i]);
     }
@@ -337,7 +346,7 @@ camera_get_image(pgCameraObject *self, PyObject *arg)
 {
 #if defined(__linux__)
     SDL_Surface *surf = NULL;
-    PyObject *surfobj = NULL;
+    pgSurfaceObject *surfobj = NULL;
 
     if (!PyArg_ParseTuple(arg, "|O!", &pgSurface_Type, &surfobj))
         return NULL;
@@ -368,14 +377,14 @@ camera_get_image(pgCameraObject *self, PyObject *arg)
 
     if (surfobj) {
         Py_INCREF(surfobj);
-        return surfobj;
+        return (PyObject *)surfobj;
     }
     else {
-        return pgSurface_New(surf);
+        return (PyObject *)pgSurface_New(surf);
     }
 #elif defined(PYGAME_MAC_CAMERA_OLD)
     SDL_Surface *surf = NULL;
-    PyObject *surfobj = NULL;
+    pgSurfaceObject *surfobj = NULL;
 
     if (!PyArg_ParseTuple(arg, "|O!", &pgSurface_Type, &surfobj))
         return NULL;
@@ -408,10 +417,10 @@ camera_get_image(pgCameraObject *self, PyObject *arg)
 
     if (surfobj) {
         Py_INCREF(surfobj);
-        return surfobj;
+        return (PyObject *)surfobj;
     }
     else {
-        return pgSurface_New(surf);
+        return (PyObject *)pgSurface_New(surf);
     }
 #endif
     Py_RETURN_NONE;
@@ -1258,8 +1267,6 @@ uyvy_to_yuv(const void *src, void *dst, int length, SDL_PixelFormat *format)
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-/* FIXME: Seems to be grayscale and kind of dark on the OLPC XO
-          Maybe the result of a different Bayer color order on the screen? */
 /* TODO: Certainly not the most efficient way of doing this conversion. */
 void
 sbggr8_to_rgb(const void *src, void *dst, int width, int height,
@@ -1716,7 +1723,7 @@ PyMethodDef cameraobj_builtins[] = {
     {"stop", (PyCFunction)camera_stop, METH_NOARGS, DOC_CAMERASTOP},
     {"get_controls", (PyCFunction)camera_get_controls, METH_NOARGS,
      DOC_CAMERAGETCONTROLS},
-    {"set_controls", (PyCFunction)camera_set_controls, METH_KEYWORDS,
+    {"set_controls", (PyCFunction)camera_set_controls, METH_VARARGS | METH_KEYWORDS,
      DOC_CAMERASETCONTROLS},
     {"get_size", (PyCFunction)camera_get_size, METH_NOARGS, DOC_CAMERAGETSIZE},
     {"query_image", (PyCFunction)camera_query_image, METH_NOARGS,
@@ -1738,7 +1745,8 @@ PyObject* camera_getattr(PyObject* self, char* attrname) {
 }
 */
 PyTypeObject pgCamera_Type = {
-    TYPE_HEAD(NULL, 0) "Camera",
+    PyVarObject_HEAD_INIT(NULL,0)
+    "Camera",
     sizeof(pgCameraObject),
     0,
     camera_dealloc,
@@ -1797,6 +1805,10 @@ Camera(pgCameraObject *self, PyObject *arg)
     if (cameraobj) {
         cameraobj->device_name =
             (char *)malloc((strlen(dev_name) + 1) * sizeof(char));
+        if (!cameraobj->device_name) {
+            Py_DECREF(cameraobj);
+            return PyErr_NoMemory();
+        }
         strcpy(cameraobj->device_name, dev_name);
         cameraobj->camera_type = 0;
         cameraobj->pixelformat = 0;
@@ -1844,6 +1856,10 @@ Camera(pgCameraObject *self, PyObject *arg)
     if (cameraobj) {
         cameraobj->device_name =
             (char *)malloc((strlen(dev_name) + 1) * sizeof(char));
+        if (!cameraobj->device_name) {
+            Py_DECREF(cameraobj);
+            return PyErr_NoMemory();
+        }
         strcpy(cameraobj->device_name, dev_name);
         if (color) {
             if (!strcmp(color, "YUV")) {
